uniform sampler2D base_tex_unit;   // base texture unit
uniform sampler2D normal_tex_unit; // normalmap texture unit

varying vec4 l1_dir;
varying vec4 l2_dir;
varying vec3 eye_dir;

varying float can_reflect_l1;
varying float can_reflect_l2;

varying vec2 tex_coord;

void main(void)
{
 vec4 bac=texture2D(base_tex_unit,tex_coord);                  // base color

 vec4 tex_normal=texture2D(normal_tex_unit,tex_coord)*2.0-1.0; // normalmap color

 // light1
 float dir=dot(l1_dir,l1_dir);
 l1_dir=l1_dir*inversesqrt(dir); // normalize
 float att1=clamp(1.0-0.04*sqrt(dir),0.0,1.0); // attenuation

 float d1=max(dot(l1_dir,tex_normal),0.0);

 // light2
 dir=dot(l2_dir,l2_dir);
 l2_dir=l2_dir*inversesqrt(dir); // normalize
 float att2=clamp(1.0-0.04*sqrt(dir),0.0,1.0); // attenuation

 float d2=max(dot(l2_dir,tex_normal),0.0);

 eye_dir=normalize(eye_dir);
 float spec1=pow(clamp(dot(reflect(-l1_dir,tex_normal),eye_dir),0.0,1.0),16.0);
 float spec2=pow(clamp(dot(reflect(-l2_dir,tex_normal),eye_dir),0.0,1.0),16.0);

 gl_FragColor=(d1*bac+1.5*spec1*clamp(can_reflect_l1,0.0,1.0))*att1+
              (d2*bac+1.5*spec2*clamp(can_reflect_l2,0.0,1.0))*att2;
}